'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var methods = {
  DEL: 'DEL',
  GET: 'GET',
  POST: 'POST',
  PUT: 'PUT'
};

var status = {
  REQUEST: 'REQUEST',
  FAILURE: 'FAILURE',
  SUCCESS: 'SUCCESS'
};

var types = {
  methods: methods,
  status: status
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var statusArr = Object.values(status);
var methodsArr = Object.values(methods);
var REQUEST = status.REQUEST,
    FAILURE = status.FAILURE,
    SUCCESS = status.SUCCESS;


var setAction = function setAction(type) {
  var payload = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return _extends({ type: type }, payload);
};

var getEntities = function getEntities(entitiesArr) {
  var idKey = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'id';

  var entities = {};

  entitiesArr.forEach(function (data) {
    entities[data[idKey]] = data;
  });

  return entities;
};

var generateTypes = function generateTypes(base) {
  var types$$1 = {};

  methodsArr.forEach(function (method) {
    statusArr.forEach(function (status$$1) {
      types$$1[method + '_' + status$$1] = base + '_' + method + '_' + status$$1;
    });
  });

  return types$$1;
};

var generateActions = function generateActions(types$$1) {
  var result = {};

  methodsArr.forEach(function (method) {
    var methodActions = {};

    statusArr.forEach(function (status$$1) {
      methodActions[status$$1.toLowerCase()] = function (payload) {
        return setAction(types$$1[method + '_' + status$$1], { payload: payload });
      };
    });

    result[method.toLowerCase()] = methodActions;
  });

  return result;
};

var createReducer = function createReducer() {
  var initialState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var handlers = arguments[1];
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments[1];
    return handlers.hasOwnProperty(action.type) ? handlers[action.type](state, action) : state;
  };
};

var generateReducer = function generateReducer(types$$1) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var initialState = _ref.initialState,
      pure = _ref.pure,
      options = objectWithoutProperties(_ref, ['initialState', 'pure']);

  var reducer = {};

  methodsArr.forEach(function (method) {
    // Request
    reducer[types$$1[method + '_' + REQUEST]] = function (state, action) {
      return _extends({}, state, {
        loaded: false,
        loading: true,
        errors: null
      }, options[method + '_' + REQUEST] ? options[method + '_' + REQUEST]({ state: state, action: action }) || {} : {});
    };
    // Success
    reducer[types$$1[method + '_' + SUCCESS]] = function (state, action) {
      return _extends({}, state, {
        loaded: true,
        loading: false,
        errors: null,
        data: pure ? action.payload : getEntities(action.payload)
      }, options[method + '_' + SUCCESS] ? options[method + '_' + SUCCESS]({ state: state, action: action }) || {} : {});
    };
    // Failure
    reducer[types$$1[method + '_' + FAILURE]] = function (state, action) {
      return _extends({}, state, {
        loaded: false,
        loading: false,
        errors: action.payload
      }, options[method + '_' + FAILURE] ? options[method + '_' + FAILURE]({ state: state, action: action }) || {} : {});
    };
  });

  return createReducer(initialState, reducer);
};

var generateSelector = function generateSelector(type) {
  return function (state) {
    return state[type.toLowerCase()];
  };
};

var generateStoreItem = function generateStoreItem(type, options) {
  var types$$1 = generateTypes(type);

  return {
    // Actions
    actions: generateActions(types$$1),
    // Reducer
    reducer: generateReducer(types$$1, options),
    // Selector
    selector: generateSelector(type),
    // Type
    types: types$$1
  };
};

var generateStoreItems = function generateStoreItems(typesArr) {
  var items = {
    actions: {},
    reducers: {},
    selectors: {},
    types: {}
  };

  typesArr.forEach(function (type) {
    var _generateStoreItem = generateStoreItem(type),
        actions = _generateStoreItem.actions,
        reducer = _generateStoreItem.reducer,
        selector = _generateStoreItem.selector,
        types$$1 = _generateStoreItem.types;

    items.actions[type.toLowerCase()] = actions;
    items.reducers[type.toLowerCase()] = reducer;
    items.selectors[type.toLowerCase()] = selector;
    items.types[type] = types$$1;
  });

  return items;
};

exports.types = types;
exports.generateStoreItem = generateStoreItem;
exports.generateStoreItems = generateStoreItems;
//# sourceMappingURL=index.js.map
